package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProdMouldStatisticsService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.dao.IMesProdMouldStatisticsDao;
import cn.estsh.i3plus.mes.pcn.util.DateUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesMouldMappingCfg;
import cn.estsh.i3plus.pojo.mes.bean.MesProdMouldRecord;
import cn.estsh.i3plus.pojo.mes.bean.MesProdMouldStatistics;
import cn.estsh.i3plus.pojo.mes.bean.MesProductionRecord;
import cn.estsh.i3plus.pojo.mes.repository.IMesMouldMappingCfgRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProdMouldRecordRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProdMouldStatisticsRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProductionRecordRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Description : 生产开模统计
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/6/26 16:32
 * @Modify:
 **/
@Service
@Slf4j
public class MesProdMouldStatisticsServiceImpl implements IMesProdMouldStatisticsService {

    @Autowired
    private IMesProdMouldStatisticsDao mesProdMouldStatisticsDao;

    @Autowired
    private IMesMouldMappingCfgRepository mesMouldMappingCfgRDao;

    @Autowired
    private MesProductionRecordRepository mesProductionRecordRDao;

    @Autowired
    private MesProdMouldRecordRepository mesProdMouldRecordRDao;

    @Autowired
    private MesProdMouldStatisticsRepository mesProdMouldStatisticsRDao;

    @Override
    public void doSumMesProdMouldStatistics(String organizeCode, Integer pageSize, String userName) {
        //查询未统计的开模记录数据
        List<MesProdMouldRecord> undStatisticsMesProdMouldRecordList = mesProdMouldStatisticsDao.findUndStatisticsMesProdMouldRecord(organizeCode, pageSize);
        if (CollectionUtils.isEmpty(undStatisticsMesProdMouldRecordList)) {
            return;
        }
        Map<String, List<MesProdMouldRecord>> undStatisticsMesProdMouldRecordMap = undStatisticsMesProdMouldRecordList.stream().collect(Collectors.groupingBy(MesProdMouldRecord::getEquipmentCode));
        //数据处理
        for (MesMouldMappingCfg mesMouldMappingCfg : getMesMouldMappingCfgs(organizeCode, new ArrayList<>(undStatisticsMesProdMouldRecordMap.keySet()))) {
            List<MesProdMouldRecord> mesProdMouldRecords = undStatisticsMesProdMouldRecordMap.get(mesMouldMappingCfg.getEquipmentCode());
            //更新开模记录数据
            mesProdMouldRecords.forEach(mesProdMouldRecord -> {
                mesProdMouldRecord.setAssetNum(mesMouldMappingCfg.getAssetNum());
                mesProdMouldRecord.setMeterName(mesMouldMappingCfg.getMeterName());
                mesProdMouldRecord.setSystemSyncStatus(CommonEnumUtil.FALSE);
                ConvertBean.serviceModelUpdate(mesProdMouldRecord, userName);
            });
            mesProdMouldRecordRDao.saveAll(mesProdMouldRecords);
            //生产开模统计
            saveMesProdMouldStatistics(organizeCode, userName, mesMouldMappingCfg, mesProdMouldRecords);
        }
        //更新加工记录表状态
        updateMesProductionRecord(organizeCode, userName, undStatisticsMesProdMouldRecordList.stream().map(MesProdMouldRecord::getId).collect(Collectors.toList()));
    }

    private void updateMesProductionRecord(String organizeCode, String userName, List<Long> idList) {
        //查询加工记录表
        List<MesProductionRecord> mesProductionRecordList = getProductionRecordList(organizeCode, idList);
        //更新加工记录表状态
        if (!CollectionUtils.isEmpty(mesProductionRecordList)) {
            List<Long> noUpdateSyncStatusId = mesProductionRecordList.stream().filter(t -> MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue() == t.getReportStatus()).map(MesProductionRecord::getId).collect(Collectors.toList());
            List<Long> updateSyncStatusId = mesProductionRecordList.stream().filter(t -> MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue() != t.getReportStatus()).map(MesProductionRecord::getId).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(noUpdateSyncStatusId)) {
                mesProdMouldStatisticsDao.updateMesProductionRecord(noUpdateSyncStatusId, organizeCode, false, userName);
            }
            if (!CollectionUtils.isEmpty(updateSyncStatusId)) {
                mesProdMouldStatisticsDao.updateMesProductionRecord(updateSyncStatusId, organizeCode, true, userName);
            }
        }
    }

    private void saveMesProdMouldStatistics(String organizeCode, String userName, MesMouldMappingCfg mesMouldMappingCfg, List<MesProdMouldRecord> mesProdMouldRecords) {
        Map<String, List<MesProdMouldRecord>> mesProdMouldRecordMap = mesProdMouldRecords.stream().collect(Collectors.groupingBy(t -> TimeTool.parseStringFormat(t.getCreateDatetime(), null, DateUtil.SHORT_FORMAT)));
        Map<String, MesProdMouldStatistics> mouldStatisticsMap = getMesProdMouldStatistics(organizeCode, mesMouldMappingCfg.getAssetNum(), mesMouldMappingCfg.getMeterName());
        for (Map.Entry<String, List<MesProdMouldRecord>> mesProdMouldRecordEntry : mesProdMouldRecordMap.entrySet()) {
            if (Objects.isNull(mouldStatisticsMap) || !mouldStatisticsMap.containsKey(mesProdMouldRecordEntry.getKey())) {
                MesProdMouldStatistics mesProdMouldStatistic = new MesProdMouldStatistics();
                mesProdMouldStatistic.setWorkDate(mesProdMouldRecordEntry.getKey());
                mesProdMouldStatistic.setAssetNum(mesMouldMappingCfg.getAssetNum());
                mesProdMouldStatistic.setMeterName(mesMouldMappingCfg.getMeterName());
                mesProdMouldStatistic.setQty(mesProdMouldRecordEntry.getValue().size());
                mesProdMouldStatistic.setOrganizeCode(organizeCode);
                ConvertBean.serviceModelInitialize(mesProdMouldStatistic, userName);
                mesProdMouldStatisticsRDao.insert(mesProdMouldStatistic);
            } else {
                MesProdMouldStatistics prodMouldStatistics = mouldStatisticsMap.get(mesProdMouldRecordEntry.getKey());
                prodMouldStatistics.setQty(prodMouldStatistics.getQty() + mesProdMouldRecordEntry.getValue().size());
                prodMouldStatistics.setSystemSyncStatus(CommonEnumUtil.FALSE);
                ConvertBean.serviceModelUpdate(prodMouldStatistics, userName);
                mesProdMouldStatisticsRDao.update(prodMouldStatistics);
            }
        }
    }

    private Map<String, MesProdMouldStatistics> getMesProdMouldStatistics(String organizeCode, String assetNum, String meterName) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(assetNum, "assetNum", packBean);
        DdlPreparedPack.getStringEqualPack(meterName, "meterName", packBean);
        List<MesProdMouldStatistics> mesProdMouldStatistics = mesProdMouldStatisticsRDao.findByHqlWhere(packBean);
        if (!CollectionUtils.isEmpty(mesProdMouldStatistics)) {
            return mesProdMouldStatistics.stream().collect(Collectors.toMap(MesProdMouldStatistics::getWorkDate, t -> t));
        }
        return null;
    }

    private List<MesMouldMappingCfg> getMesMouldMappingCfgs(String organizeCode, List<String> equipmentCodeList) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(equipmentCodeList, "equipmentCode", packBean);
        return mesMouldMappingCfgRDao.findByHqlWhere(packBean);
    }

    private List<MesProductionRecord> getProductionRecordList(String organizeCode, List<Long> idList) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(idList, "mouldRecordId", packBean);
        return mesProductionRecordRDao.findByHqlWhere(packBean);
    }
}
