package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesPartService;
import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesTemplateService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProduceSnPrintService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesProduceSnPrintDataModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesProduceSnPrintModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.api.iservice.base.IConfigService;
import cn.estsh.i3plus.mes.pcn.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.mes.pcn.util.DateUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.codemaker.SnowflakeIdMaker;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.hutool.core.bean.BeanUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Description : 条码补打
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/6/13 15:52
 * @Modify:
 **/
@Service
public class MesProduceSnPrintService implements IMesProduceSnPrintService {

    @Autowired
    private MesPackingDefineDetailsRepository mesPackingDefineDetailsRDao;

    @Autowired
    private MesPackingDefineRepository mesPackingDefineRDao;

    @Autowired
    private MesProduceSnRepository mesProduceSnRDao;

    @Autowired
    private SnowflakeIdMaker snowflakeIdMaker;

    @Autowired
    private IMesPartService mesPartService;

    @Autowired
    private IMesTemplateService mesTemplateService;

    @Autowired
    private MesCustomerPartRepository mesCustomerPartRDao;


    @Autowired
    private IConfigService configService;

    @Autowired
    private ISyncFuncService syncFuncService;

    @Autowired
    private MesNumberRuleRepository mesNumberRuleRDao;

    @Autowired
    private IMesPrintedSnLogRepository mesPrintedSnRao;

    @Override
    public MesProduceSnPrintModel findMesPackingDefine(MesProduceSnPrintModel mesProduceSnPrintModel) {
        //查询SAP下发物料信息
        MesPartSap mesPartSap = mesPartService.getMesPartSapByPartNo(mesProduceSnPrintModel.getPartNo(), mesProduceSnPrintModel.getOrganizeCode());
        mesProduceSnPrintModel.setQty(mesPartSap.getSnp());
        mesProduceSnPrintModel.setPartName(mesPartSap.getPartName());
        //查询包装定义明细
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesProduceSnPrintModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProduceSnPrintModel.getPartNo(), "partNo", ddlPackBean);
        List<MesPackingDefineDetails> packingDefineDetails = mesPackingDefineDetailsRDao.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(packingDefineDetails)) {
            MesPcnException.throwMesBusiException("物料【%s】未维护包装定义信息", mesProduceSnPrintModel.getPartNo());
        }
        //查询包装定义主表
        ddlPackBean = DdlPackBean.getDdlPackBean(mesProduceSnPrintModel.getOrganizeCode());
        DdlPreparedPack.getInPackList(packingDefineDetails.stream().map(MesPackingDefineDetails::getPackCode).collect(Collectors.toList()), "packCode", ddlPackBean);
        mesProduceSnPrintModel.setMesPackingDefineList(mesPackingDefineRDao.findByHqlWhere(ddlPackBean));
        return mesProduceSnPrintModel;
    }

    @Override
    public MesProduceSn findMesProduceSn(MesProduceSnPrintModel mesProduceSnPrintModel) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesProduceSnPrintModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProduceSnPrintModel.getProductSn(), "productSn", ddlPackBean);
        MesProduceSn mesProduceSn = mesProduceSnRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesProduceSn)) {
            MesPcnException.throwMesBusiException("条码【%s】信息不存在", mesProduceSnPrintModel.getProductSn());
        }
        return mesProduceSn;
    }

    @Override
    public MesProduceSnPrintModel doPrint(MesProduceSnPrintModel mesProduceSnPrintModel) {
        String organizeCode = mesProduceSnPrintModel.getOrganizeCode();
        // 获取工厂名称
        MesConfig mesConfig = configService.getMesConfigByCfgCode(MesPcnExtConstWords.ORGANIZE_NAME, organizeCode);
        if (null == mesConfig || StringUtils.isEmpty(mesConfig.getCfgValue())) {
            throw ImppExceptionBuilder.newInstance().setSystemID(CommonEnumUtil.SOFT_TYPE.MES_PCN.getCode()).setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode()).setErrorDetail("没有维护当前工厂的系统参数!").build();
        }
        //获取工厂名称
        String organizeName = mesConfig.getCfgValue();
        //物料信息
        MesPart mesPart = mesPartService.getMesPartByPartNo(mesProduceSnPrintModel.getPartNo(), organizeCode);
        //条码生成
        if (StringUtils.isEmpty(mesProduceSnPrintModel.getProductSn())) {
            //打印模板
            String printTemplate = mesProduceSnPrintModel.getPrintTemplate();
            //打印规则
            String snRule = mesProduceSnPrintModel.getSnRule();
            //获取条码规则,根据条码规则中的序列号长度截取序列号
            DdlPackBean mesRulePackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(snRule, "ruleCode", mesRulePackBean);
            List<MesNumberRule> ruleList = mesNumberRuleRDao.findByHqlWhere(mesRulePackBean);
            if (ruleList.isEmpty()) {
                MesPcnException.throwMesBusiException("零件号【】打印规则查询不到!", mesPart.getPartNo());
            }
            MesNumberRule numberRule = ruleList.get(0);

            //打印日期格式
            String format = "yyyy/MM/dd";
            if (MesExtEnumUtil.MES_PRINT_TEMPLATE.NO_SORT_BAR_CODE_GZ.getCode().equals(printTemplate) || MesExtEnumUtil.MES_PRINT_TEMPLATE.XIAO_PENG.getCode().equals(printTemplate) || MesExtEnumUtil.MES_PRINT_TEMPLATE.XIAO_PENG_NEW.getCode().equals(printTemplate)) {
                format = DateUtil.BASE_FORMAT;
            }
            int printQty = mesProduceSnPrintModel.getPrintQty();
            Date date = new Date();

            //if 1B2M || GQAA_BARCODE
            if (MesExtEnumUtil.MES_PRINT_TEMPLATE.B2M.getCode().equals(printTemplate) || MesExtEnumUtil.MES_PRINT_TEMPLATE.GQAA.getCode().equals(printTemplate)) {
                GenSerialNoModel genSerialNoModel = new GenSerialNoModel();
                genSerialNoModel.setRuleCode(numberRule.getRuleCode());
                //${SPILTRULE}${partNo}${SPILTRULE}${SPILTRULE}${year}${month}${day}${serialNo}${SPILTRULE}
                //规格号,零件号,供应商，生产线代码,年,月,日,序列号,软件版本号
                // ----广汽 1B2M & 广新 GQAA_BARCODE 21位
                //规格号 ${SPILTRULE} +
                //零件简号 {partNo} - mesPart.getPartSnParam()
                genSerialNoModel.setPartNo(mesPart.getPartSnParam());
                //供应商号 supplierNo  ---固定VD501 ${SPILTRULE}拼接
                //生产线代码 workCenterCode 或者 fixWorkCenterCode (取配置值) ${SPILTRULE}
                //日期${dateShort}
                genSerialNoModel.setYear(getYearShort(date));
                genSerialNoModel.setMonth(getMonthShort(date));
                genSerialNoModel.setDay(getDayShort(date));
                //获取客户零件号
                String custPartNo = getMesCustomerPart(mesProduceSnPrintModel).getCustPartNo();
                //供应商生产流水号 ${serialNo} 采用34进制 获取条码是十进制,需要截取转换成34位
                //软件版本号${versionNo} ${SPILTRULE}
                for (int i = 0; i < printQty; i++) {
                    ResultBean resultBean = syncFuncService.syncSerialNo(genSerialNoModel, mesProduceSnPrintModel.getUserName(), organizeCode, 1);
                    String sn = resultBean.getResultList().get(0).toString();
                    int versionNoLength = numberRule.getSpiltRule().substring(numberRule.getSpiltRule().lastIndexOf(",") + 1).length();
                    Integer serialNoLength = numberRule.getSerialnoLength();
                    String snPrefix = sn.substring(0, sn.length() - versionNoLength - serialNoLength);
                    String snSuffix = sn.substring(sn.length() - versionNoLength);
                    String serialNo = sn.substring(sn.length() - versionNoLength - serialNoLength, sn.length() - versionNoLength);
                    String serialNo34 = toBase34(Integer.parseInt(serialNo));
                    serialNo34 = stringFormat(serialNo34, 4, false, "0");
                    sn = snPrefix + serialNo34 + snSuffix;
                    //保存条码信息
                    MesProduceSn produceSn = generateMesProduceSn(mesPart, sn, mesProduceSnPrintModel.getUserName(), mesProduceSnPrintModel.getQty(), format);
                    //封装打印信息
                    MesProduceSnPrintDataModel model = getModel(produceSn, serialNo, custPartNo, organizeName);
                    mesProduceSnPrintModel.getMesProduceSnPrintDataModelList().add(model);
                    mesProduceSnRDao.insert(produceSn);
                    //保存打印记录
                    MesPrintedSnLog snLog = new MesPrintedSnLog();
                    ConvertBean.serviceModelInitialize(snLog, mesProduceSnPrintModel.getUserName());
                    BeanUtil.copyProperties(model,snLog);
                    mesPrintedSnRao.insert(snLog);
                }

            } else if (MesExtEnumUtil.MES_PRINT_TEMPLATE.GQXNY.getCode().equals(printTemplate)) {

                // else if GQXNY
                //----广乘 45位 GQXNY
                //{custPartNo}{SPILTURE}{custCode}{spiltrure}{spiltrule}{year}{serialNo}{spilture}{spiltrule}
                //客户零件号-扩展位-客户代码-预留位000-产线1-年月日-条码-扩展0000-追溯位+
                //零件长号---17位,不足17位 右边补*
                GenSerialNoModel genSerialNoModel = new GenSerialNoModel();
                genSerialNoModel.setRuleCode(numberRule.getRuleCode());
                //获取客户零件号
                String custPartNo = getMesCustomerPart(mesProduceSnPrintModel).getCustPartNo();
                String custPartNoFormat = stringFormat(custPartNo, 17);
                genSerialNoModel.setCustPartNo(custPartNoFormat);
                //拓展位 2位 00 包含0~9,A~Z 不使用时，补“0”填充 {SPILTRULE}
                //供应商代码 7位 供应商代码 0~9,A~Z;不足7位 右测补*
                MesConfig supplierConfig = configService.getMesConfigByCfgCodeKeyOrg(MesPcnExtConstWords.SUPPLIER_CODE, MesExtEnumUtil.MES_PRINT_TEMPLATE.GQXNY.getValue(), organizeCode);
                String supplierCode = supplierConfig.getCfgValue();
                String supplierCodeFormat = stringFormat(supplierCode, 7);
                genSerialNoModel.setCustCode(supplierCodeFormat);
                //扩展位 3位 预留扩展位0~9 A~Z 不使用时，补 0 {SPILTRULE}
                //产线 1位 供应商产线号，表示由数字1~9 和字母A~Z(I/O除外)组成; {SPILTRULE}
                //年月日 6位 yyMMdd 零部件生产日期，年月日各2位
                String year = TimeTool.pareDateToString("yyMMdd", date);
                genSerialNoModel.setYear(year);
                //流水号 4位 采用34进制 0001~ZZZZ (I、O除外)顺位进行标注 (同广汽 & 广新) {serialNo}
                //扩展位 4位 预留扩展位信息，包含0~9和字母A~Z,不使用时，补“0” 填充;  {SPILTRULE}
                //追溯标识 1位 + 代表精确按件追溯，$标识按批追溯, % 代表按生产方式追溯  {SPILTRULE}
                for (int i = 0; i < printQty; i++) {
                    ResultBean resultBean = syncFuncService.syncSerialNo(genSerialNoModel, mesProduceSnPrintModel.getUserName(), organizeCode, 1);
                    String sn = resultBean.getResultList().get(0).toString();
                    String snPrefix = sn.substring(0, sn.length() - 12);
                    String snSuffix = sn.substring(sn.length() - 5);
                    String serialNo = sn.substring(sn.length() - 12, sn.length() - 5);
                    String serialNo34 = toBase34(Integer.parseInt(serialNo));
                    serialNo34 = stringFormat(serialNo34, 4, false, "0");
                    sn = snPrefix + serialNo34 + snSuffix;
                    //保存条码信息
                    MesProduceSn produceSn = generateMesProduceSn(mesPart, sn, mesProduceSnPrintModel.getUserName(), mesProduceSnPrintModel.getQty(), format);
                    //封装打印信息
                    MesProduceSnPrintDataModel model = getModel(produceSn, serialNo, custPartNo, organizeName);
                    model.setSupplierCode(supplierCode);
                    mesProduceSnPrintModel.getMesProduceSnPrintDataModelList().add(model);
                    mesProduceSnRDao.insert(produceSn);
                    //保存打印记录
                    MesPrintedSnLog snLog = new MesPrintedSnLog();
                    ConvertBean.serviceModelInitialize(snLog, mesProduceSnPrintModel.getUserName());
                    BeanUtil.copyProperties(model,snLog);
                    mesPrintedSnRao.insert(snLog);
                }

            } else if (MesExtEnumUtil.MES_PRINT_TEMPLATE.NO_SORT_BAR_CODE_GZ.getCode().equals(printTemplate)) {
                // noSortBarCodeGz
                //{partNo}{year}{month}{day}{serialNo}
                //内部条码, 批量打印 NoSortBarCodeGZ
                //零件号 partNo
                String partNo = mesPart.getPartNo();
                GenSerialNoModel genSerialNoModel = new GenSerialNoModel();
                genSerialNoModel.setRuleCode(numberRule.getRuleCode());
                genSerialNoModel.setPartNo(partNo);
                //年月日 yyyyMMdd
                //流水号 4位 每天从1开始
                for (int i = 0; i < printQty; i++) {
                    ResultBean resultBean = syncFuncService.syncSerialNo(genSerialNoModel, mesProduceSnPrintModel.getUserName(), organizeCode, 1);
                    String sn = resultBean.getResultList().get(0).toString();
                    Integer serialNoLength = numberRule.getSerialnoLength();
                    String serialNo = sn.substring(sn.length() - serialNoLength);
                    //保存条码信息
                    MesProduceSn produceSn = generateMesProduceSn(mesPart, sn, mesProduceSnPrintModel.getUserName(), mesProduceSnPrintModel.getQty(), format);
                    //封装打印信息
                    MesProduceSnPrintDataModel model = getModel(produceSn, serialNo, mesProduceSnPrintModel.getCustPartNo(), organizeName);
                    mesProduceSnPrintModel.getMesProduceSnPrintDataModelList().add(model);
                    mesProduceSnRDao.insert(produceSn);
                    //保存打印记录
                    MesPrintedSnLog snLog = new MesPrintedSnLog();
                    ConvertBean.serviceModelInitialize(snLog, mesProduceSnPrintModel.getUserName());
                    BeanUtil.copyProperties(model,snLog);
                    mesPrintedSnRao.insert(snLog);
                }
            } else if (MesExtEnumUtil.MES_PRINT_TEMPLATE.XIAO_PENG.getCode().equals(printTemplate)) {
                format = DateUtil.BASE_FORMAT;
                // XIAO_PENG 原客户条码.小鹏汽车 NoSortBarCodeGZ-new
                //{custPartNo}{SPILTRULE}{year}{month}{day}{hour}{minute}{second}{serialNo}
                GenSerialNoModel genSerialNoModel = new GenSerialNoModel();
                genSerialNoModel.setRuleCode(numberRule.getRuleCode());
                //客户零件号
                String custPartNo = getMesCustomerPart(mesProduceSnPrintModel).getCustPartNo();
                genSerialNoModel.setCustPartNo(custPartNo);
                //日期${dateShort}
                genSerialNoModel.setYear(getYearShort(date));
                genSerialNoModel.setMonth(getMonthShort(date));
                genSerialNoModel.setDay(getDayShort(date));
                //中杠 -
                //生产日期 + 时间 yyyyMMddHHmmss
                //流水号 5位 每天从1开始
                for (int i = 0; i < printQty; i++) {
                    ResultBean resultBean = syncFuncService.syncSerialNo(genSerialNoModel, mesProduceSnPrintModel.getUserName(), organizeCode, 1);
                    String sn = resultBean.getResultList().get(0).toString();
                    Integer serialNoLength = numberRule.getSerialnoLength();
                    String serialNo = sn.substring(sn.length() - serialNoLength);
                    //保存条码信息
                    MesProduceSn produceSn = generateMesProduceSn(mesPart, sn, mesProduceSnPrintModel.getUserName(), mesProduceSnPrintModel.getQty(), format);
                    //封装打印信息
                    MesProduceSnPrintDataModel model = getModel(produceSn, serialNo, mesProduceSnPrintModel.getCustPartNo(), organizeName);
                    mesProduceSnPrintModel.getMesProduceSnPrintDataModelList().add(model);
                    mesProduceSnRDao.insert(produceSn);
                    //保存打印记录
                    MesPrintedSnLog snLog = new MesPrintedSnLog();
                    ConvertBean.serviceModelInitialize(snLog, mesProduceSnPrintModel.getUserName());
                    BeanUtil.copyProperties(model,snLog);
                    mesPrintedSnRao.insert(snLog);
                }

            } else if (MesExtEnumUtil.MES_PRINT_TEMPLATE.XIAO_PENG_NEW.getCode().equals(printTemplate)) {
                format = "yyyy-MM-dd";
                //gz-xiaopeng
                //客户条码-小鹏汽车 gz-xiaopeng
                //{custPartNo}{year}{month}{day}{serialNo}
                GenSerialNoModel genSerialNoModel = new GenSerialNoModel();
                //客户零件号
                String custPartNo = getMesCustomerPart(mesProduceSnPrintModel).getCustPartNo();
                genSerialNoModel.setCustPartNo(custPartNo);
                //年月日缩写
                genSerialNoModel.setYear(getYearShort(date));
                genSerialNoModel.setMonth(getMonthShort(date));
                genSerialNoModel.setDay(getDayShort(date));
                genSerialNoModel.setRuleCode(numberRule.getRuleCode());
                //流水号 5位 每天从1开始
                for (int i = 0; i < printQty; i++) {
                    ResultBean resultBean = syncFuncService.syncSerialNo(genSerialNoModel, mesProduceSnPrintModel.getUserName(), organizeCode, 1);
                    String sn = resultBean.getResultList().get(0).toString();
                    Integer serialNoLength = numberRule.getSerialnoLength();
                    String serialNo = sn.substring(sn.length() - serialNoLength);
                    //保存条码信息
                    MesProduceSn produceSn = generateMesProduceSn(mesPart, sn, mesProduceSnPrintModel.getUserName(), mesProduceSnPrintModel.getQty(), format);
                    produceSn.setCustPartNo(custPartNo);
                    //封装打印信息
                    MesProduceSnPrintDataModel model = getModel(produceSn, serialNo, custPartNo, organizeName);
                    mesProduceSnPrintModel.getMesProduceSnPrintDataModelList().add(model);
                    mesProduceSnRDao.insert(produceSn);
                    //保存打印记录
                    MesPrintedSnLog snLog = new MesPrintedSnLog();
                    ConvertBean.serviceModelInitialize(snLog, mesProduceSnPrintModel.getUserName());
                    BeanUtil.copyProperties(model,snLog);
                    mesPrintedSnRao.insert(snLog);
                }
            }
            //条码补打
        } else {
            //查找打印记录
            DdlPackBean snLogPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(mesProduceSnPrintModel.getProductSn(),"barcode",snLogPackBean);
            List<MesPrintedSnLog> snLogList = mesPrintedSnRao.findByHqlWhere(snLogPackBean);
            if (snLogList.isEmpty()){
                MesPcnException.throwMesBusiException("条码【】查询不到打印记录!", mesProduceSnPrintModel.getProductSn());
            }
            MesPrintedSnLog mesPrintedSnLog = snLogList.get(0);
            MesProduceSnPrintDataModel model = new MesProduceSnPrintDataModel();
            BeanUtil.copyProperties(mesPrintedSnLog,model);
            mesProduceSnPrintModel.getMesProduceSnPrintDataModelList().add(model);
        }
        //获取打印模板信息
        mesProduceSnPrintModel.setMesLabelTemplate(mesTemplateService.getMesLabelTemplate(mesProduceSnPrintModel.getPrintTemplate()));
        return mesProduceSnPrintModel;
    }

    private String getTemplate(MesProduceSnPrintModel mesProduceSnPrintModel, MesPart mesPart) {
        Date date = new Date();
        StringBuilder template = new StringBuilder();
        if (MesPcnEnumUtil.PRINT_TEMPLATE.B2M.getCode().equals(mesProduceSnPrintModel.getPrintTemplate()) || MesPcnEnumUtil.PRINT_TEMPLATE.GQAA.getCode().equals(mesProduceSnPrintModel.getPrintTemplate())) {
            //获取客户零件号
            MesCustomerPart mesCustomerPart = getMesCustomerPart(mesProduceSnPrintModel);
            mesProduceSnPrintModel.setCustPartNo(mesCustomerPart.getCustPartNo());
            //"+"（1）+零件简号（4）+ 供应商代码（SE103）+生产线代号(1)+生产日期(4)+供应商生产流水号(4) +软件版本代号(2)
            template.append("+").append(mesPart.getPartSnParam()).append("SE103").append("1").append(getDateTime(date)).append("serialNo").append("01");
        } else if (MesPcnEnumUtil.PRINT_TEMPLATE.GQXNY.getCode().equals(mesProduceSnPrintModel.getPrintTemplate())) {
            //获取客户零件号
            MesCustomerPart mesCustomerPart = getMesCustomerPart(mesProduceSnPrintModel);
            mesProduceSnPrintModel.setCustPartNo(mesCustomerPart.getCustPartNo());
            //零件号（17）+扩展位（2）+供应商代码（7）+扩展位（3）+产线（1）+年月日（6位）+流水号（4）+扩展位（4）+"+"
            template.append(stringFormat(mesCustomerPart.getCustPartNo(), 17)).append("00").append(stringFormat("VD501", 7)).append("000").append("1").append(TimeTool.pareDateToString("yyMMdd", date)).append("serialNo").append("0000").append("+");
        } else if (MesPcnEnumUtil.PRINT_TEMPLATE.NO_SORT_BAR_CODE_GZ.getCode().equals(mesProduceSnPrintModel.getPrintTemplate())) {
            //内部条码：M00061102-R619+"10158"
            //条码拼接规则：零件号+年月日+5位流水号
            template.append(mesPart.getPartNo()).append("-").append(getDateTime(date)).append("serialNo");
        } else if (MesPcnEnumUtil.PRINT_TEMPLATE.NO_SORT_BAR_CODE_GZ_NEW.getCode().equals(mesProduceSnPrintModel.getPrintTemplate())) {
            //获取客户零件号
            MesCustomerPart mesCustomerPart = getMesCustomerPart(mesProduceSnPrintModel);
            mesProduceSnPrintModel.setCustPartNo(mesCustomerPart.getCustPartNo());
            //小鹏客户条码：6205100EF3-S5-06-R61910027
            //条码拼接规则：客户零件号+中杠+年月日+5位流水号（按天递增）
            template.append(mesCustomerPart.getCustPartNo()).append("-").append(getDateTime(date)).append("serialNo");
        }
        return template.toString();
    }

    private MesProduceSnPrintDataModel getModel(MesProduceSn produceSn, String no, String custPartNo, String factoryName) {
        MesProduceSnPrintDataModel mesProduceSnPrintDataModel = new MesProduceSnPrintDataModel();
        mesProduceSnPrintDataModel.setFactoryName(factoryName);
        mesProduceSnPrintDataModel.setPartNo(produceSn.getPartNo());
        mesProduceSnPrintDataModel.setPartName(produceSn.getPartName());
        mesProduceSnPrintDataModel.setNo(no);
        mesProduceSnPrintDataModel.setBarcode(produceSn.getProductSn());
        mesProduceSnPrintDataModel.setCustPartNo(custPartNo);
        mesProduceSnPrintDataModel.setProductDate(produceSn.getLotNo());
        return mesProduceSnPrintDataModel;
    }

//    private List<String> getSerialNoList(MesProduceSnPrintModel mesProduceSnPrintModel) {
//        List<String> serialNoList = new ArrayList<>();
//        //获取系统参数
//        String configKey = MesPcnEnumUtil.PRINT_TEMPLATE.codeOfValue(mesProduceSnPrintModel.getPrintTemplate());
//        if (StringUtils.isEmpty(configKey)) {
//            MesPcnException.throwMesBusiException("打印模板未匹配到流水码规则");
//        }
//        MesConfig mesConfig = mesConfigService.getMesConfig(mesProduceSnPrintModel.getOrganizeCode(), configKey);
//        int cfgValue = StringUtils.isEmpty(mesConfig.getCfgValue()) || !CheckTool.isInteger(mesConfig.getCfgValue()) ? 1 : Integer.parseInt(mesConfig.getCfgValue());
//        if (MesPcnEnumUtil.PRINT_TEMPLATE.B2M.getCode().equals(mesProduceSnPrintModel.getPrintTemplate()) || MesPcnEnumUtil.PRINT_TEMPLATE.GQAA.getCode().equals(mesProduceSnPrintModel.getPrintTemplate())) {
//            if (StringUtils.isEmpty(mesConfig.getCfgValueDesc()) || !TimeTool.getYear().equals(mesConfig.getCfgValueDesc())) {
//                cfgValue = 1;
//                mesConfig.setCfgValueDesc(TimeTool.getYear());
//            }
//            for (int i = 0; i < mesProduceSnPrintModel.getPrintQty(); i++) {
//                serialNoList.add(stringFormat(toBase34(cfgValue), 4, false, "0"));
//                cfgValue++;
//            }
//        } else if (MesPcnEnumUtil.PRINT_TEMPLATE.GQXNY.getCode().equals(mesProduceSnPrintModel.getPrintTemplate())) {
//            for (int i = 0; i < mesProduceSnPrintModel.getPrintQty(); i++) {
//                serialNoList.add(String.format("%04d", cfgValue));
//                cfgValue++;
//            }
//        } else if (MesPcnEnumUtil.PRINT_TEMPLATE.NO_SORT_BAR_CODE_GZ.getCode().equals(mesProduceSnPrintModel.getPrintTemplate()) || MesPcnEnumUtil.PRINT_TEMPLATE.NO_SORT_BAR_CODE_GZ_NEW.getCode().equals(mesProduceSnPrintModel.getPrintTemplate())) {
//            if (StringUtils.isEmpty(mesConfig.getCfgValueDesc()) || !TimeTool.getToday().equals(mesConfig.getCfgValueDesc())) {
//                cfgValue = 1;
//                mesConfig.setCfgValueDesc(TimeTool.getToday());
//            }
//            for (int i = 0; i < mesProduceSnPrintModel.getPrintQty(); i++) {
//                serialNoList.add(String.format("%05d", cfgValue));
//                cfgValue++;
//            }
//        }
//        mesConfig.setCfgValue(String.valueOf(cfgValue));
//        ConvertBean.serviceModelUpdate(mesConfig, mesProduceSnPrintModel.getUserName());
//        mesConfigRDao.update(mesConfig);
//        return serialNoList;
//    }

    private MesCustomerPart getMesCustomerPart(MesProduceSnPrintModel mesProduceSnPrintModel) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesProduceSnPrintModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProduceSnPrintModel.getPartNo(), "erpPartNo", ddlPackBean);
        MesCustomerPart mesCustomerPart = mesCustomerPartRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesCustomerPart)) {
            MesPcnException.throwMesBusiException("物料【%s】客户零件关系未维护", mesProduceSnPrintModel.getPartNo());
        }
        return mesCustomerPart;
    }

    private MesProduceSn generateMesProduceSn(MesPart mesPart, String sn, String userName, Double qty, String format) {
        MesProduceSn mesProduceSn = new MesProduceSn();
        mesProduceSn.setSerialNumber(snowflakeIdMaker.nextId() + "");
        mesProduceSn.setProductSn(sn);
        mesProduceSn.setCustSn(sn);
        mesProduceSn.setPartNo(mesPart.getPartNo());
        mesProduceSn.setPartName(mesPart.getPartName());
        mesProduceSn.setProcessLabelTemplate(mesPart.getProcessLabelTemplate());
        mesProduceSn.setCustLabelTemplate(mesPart.getCustLabelTemplate());
        mesProduceSn.setProdLabelTemplate(mesPart.getProductLabelTemplate());
        mesProduceSn.setQty(qty);
        mesProduceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.CREATE.getValue());
        mesProduceSn.setQcStatus(MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue());
        mesProduceSn.setLotNo(TimeTool.getNowTime(format));
        mesProduceSn.setPrintCount(MesPcnExtConstWords.ONE);
        mesProduceSn.setPrintStatus(MesExtEnumUtil.PRINT_STATUS.PRINTED.getValue());
        mesProduceSn.setOrganizeCode(mesPart.getOrganizeCode());
        ConvertBean.serviceModelInitialize(mesProduceSn, userName);
        return mesProduceSn;
    }

    private String getDateTime(Date date) {
        StringBuilder dateTime = new StringBuilder();
        String yearStr = MesExtEnumUtil.YEAR_SHORT.valueOfDescription(Integer.parseInt(TimeTool.getYear(date)));
        if (StringUtils.isEmpty(yearStr)) {
            MesPcnException.throwMesBusiException("【%s】年月简号不存在", TimeTool.getYear(date));
        }
        String monthStr = MesExtEnumUtil.MONTH_SHORT.valueOfDescription(Integer.parseInt(TimeTool.getMonth(date)));
        if (StringUtils.isEmpty(monthStr)) {
            MesPcnException.throwMesBusiException("【%s】月简号不存在", TimeTool.getMonth(date));
        }
        return dateTime.append(yearStr).append(monthStr).append(TimeTool.getDay(date)).toString();
    }

    private String getYearShort(Date date) {
        String yearStr = MesExtEnumUtil.YEAR_SHORT.valueOfDescription(Integer.parseInt(TimeTool.getYear(date)));
        if (StringUtils.isEmpty(yearStr)) {
            MesPcnException.throwMesBusiException("【%s】年月简号不存在", TimeTool.getYear(date));
        }
        return yearStr;
    }

    private String getMonthShort(Date date) {
        String monthStr = MesExtEnumUtil.MONTH_SHORT.valueOfDescription(Integer.parseInt(TimeTool.getMonth(date)));
        if (StringUtils.isEmpty(monthStr)) {
            MesPcnException.throwMesBusiException("【%s】月简号不存在", TimeTool.getMonth(date));
        }
        return monthStr;
    }

    private String getDayShort(Date date) {
        String day = TimeTool.getDay(date);
        return day;
    }

    private String stringFormat(String data, Integer length) {
        return stringFormat(data, length, true, "*");
    }

    private String stringFormat(String data, Integer length, boolean left, String symbol) {
        //数据长途大于填充位数直接返回
        if (data.length() >= length) {
            return data;
        }
        StringBuilder dataBuffer = new StringBuilder();
        //右拼接
        if (left) {
            dataBuffer.append(data);
            for (int i = 0; i < length - data.length(); i++) {
                dataBuffer.append(symbol);
            }
        }

        //左拼接
        if (!left) {
            for (int i = 0; i < length - data.length(); i++) {
                dataBuffer.append(symbol);
            }
            dataBuffer.append(data);
        }
        return dataBuffer.toString();
    }

    // 34进制字符集
    private String toBase34(int number) {
        String base34Chars = "0123456789ABCDEFGHJKLMNPQRSTUVWXYZ";
        StringBuilder sb = new StringBuilder();

        while (number > 0) {
            int remainder = number % 34;
            sb.append(base34Chars.charAt(remainder));
            number /= 34;
        }
        return sb.reverse().toString();
    }
}
